
#region Using declarations
using NinjaTrader.Cbi;
using NinjaTrader.Gui;
using NinjaTrader.Gui.Chart;
using SharpDX.Direct2D1;
using System.ComponentModel;
using System.ComponentModel.DataAnnotations;
using System.Xml.Serialization;
#endregion

//This namespace holds Indicators in this folder and is required. Do not change it.
namespace NinjaTrader.NinjaScript.Indicators.j2
{
    public class j2Watermark : Indicator
{

		private WaterMarkLocation waterMarkLocation;
		private WaterMarkStyle waterMarkStyle;
		private string sWaterMarkText;

		private System.Windows.Media.Brush	textBrush;

		protected override void OnStateChange()
		{
			if (State == State.SetDefaults)
			{
				Description					= @"Display Watermark on Chart";
				Name						= "j2Watermark";
				Calculate					= Calculate.OnBarClose;
				DisplayInDataBox			= false;
				IsOverlay					= true;
				IsChartOnly					= true;
				IsSuspendedWhileInactive	= true;
				ScaleJustification			= ScaleJustification.Right;
				WaterMarkLocation			= WaterMarkLocation.TopLeft;
				WaterMarkStyle				= WaterMarkStyle.Custom;
				WMBrush						= System.Windows.Media.Brushes.White;
				WMSize						= 225;
				WMOpacity					= 20;
				CustomText					= "j2";
			}
			else if (State == State.Historical)
			{
				SetZOrder(-1); 
			}
			else if (State == State.Configure)
			{
				if (waterMarkStyle == WaterMarkStyle.Custom)
				{
					sWaterMarkText = CustomText;
				}
				else if (waterMarkStyle == WaterMarkStyle.Description)
				{
					sWaterMarkText = Instrument.MasterInstrument.Description;
				}
				else if (waterMarkStyle == WaterMarkStyle.SymbolAndExpiry)
				{
					sWaterMarkText = Instrument.FullName;
				}
				else
				{
					sWaterMarkText = Instrument.MasterInstrument.Name;
				}
			}
		}

		protected override void OnRender(ChartControl chartControl, ChartScale chartScale)
		{


			if (!IsInHitTest)
			{
				SharpDX.Vector2 vTopLeft = new SharpDX.Vector2(ChartPanel.X, ChartPanel.Y);
				SharpDX.Vector2 vBottomRight = new SharpDX.Vector2(ChartPanel.X + ChartPanel.W, ChartPanel.Y + ChartPanel.H);

				SharpDX.Vector2 vBottomLeft = new SharpDX.Vector2(ChartPanel.X, ChartPanel.Y + ChartPanel.H);
				SharpDX.Vector2 vTopRight = new SharpDX.Vector2(ChartPanel.X + ChartPanel.W, ChartPanel.Y);

				SharpDX.Vector2 vCenter = (vTopLeft + vBottomRight) / 2;

				SharpDX.Direct2D1.Brush textBrushDx;
				textBrushDx = textBrush.ToDxBrush(RenderTarget);
				textBrushDx.Opacity = (float)WMOpacity/100;

				SharpDX.Direct2D1.AntialiasMode oldAntialiasMode = RenderTarget.AntialiasMode;

				RenderTarget.AntialiasMode = SharpDX.Direct2D1.AntialiasMode.PerPrimitive;



				SharpDX.DirectWrite.TextFormat textFormat =
					new SharpDX.DirectWrite.TextFormat(Core.Globals.DirectWriteFactory, ChartControl.Properties.LabelFont.FamilySerialize, WMSize);
				SharpDX.DirectWrite.TextLayout textLayout =
					new SharpDX.DirectWrite.TextLayout(NinjaTrader.Core.Globals.DirectWriteFactory, sWaterMarkText, textFormat, ChartPanel.W, ChartPanel.H);

				SharpDX.Vector2 TextPoint;

				switch (waterMarkLocation)
				{
					case WaterMarkLocation.Center:
						{
							TextPoint = new SharpDX.Vector2(vCenter.X - (textLayout.Metrics.Width / 2), vCenter.Y - (textLayout.Metrics.Height / 2));
							break;
						}
					case WaterMarkLocation.TopLeft:
						{
							TextPoint = new SharpDX.Vector2(vTopLeft.X, vTopLeft.Y);
							break;
						}
					case WaterMarkLocation.TopCenter:
						{
							TextPoint = new SharpDX.Vector2(vCenter.X - (textLayout.Metrics.Width / 2), vTopLeft.Y);
							break;
						}
					case WaterMarkLocation.TopRight:
						{
							TextPoint = new SharpDX.Vector2(ChartPanel.W - textLayout.Metrics.Width, vTopLeft.Y);
							break;
						}
					case WaterMarkLocation.BottomLeft:
						{
							TextPoint = new SharpDX.Vector2(vBottomLeft.X , vBottomLeft.Y - textLayout.Metrics.Height);
							break;
						}
					case WaterMarkLocation.BottomCenter:
						{
							TextPoint = new SharpDX.Vector2(vCenter.X - (textLayout.Metrics.Width / 2), vBottomLeft.Y - textLayout.Metrics.Height);
							break;
						}
					case WaterMarkLocation.BottomRight:
						{
							TextPoint = new SharpDX.Vector2(ChartPanel.W - textLayout.Metrics.Width, vBottomLeft.Y - textLayout.Metrics.Height);
							break;
						}
					default:
						{
							TextPoint = new SharpDX.Vector2(vCenter.X - (textLayout.Metrics.Width / 2), vCenter.Y - (textLayout.Metrics.Height / 2));
							break;
						}
				}

				RenderTarget.DrawTextLayout(TextPoint, textLayout, textBrushDx, SharpDX.Direct2D1.DrawTextOptions.NoSnap);

				RenderTarget.AntialiasMode = oldAntialiasMode;

				textBrushDx.Dispose();
				textFormat.Dispose();
				textLayout.Dispose();			
			}
		}

		#region Properties
		[NinjaScriptProperty]
		[Range(1, int.MaxValue)]
		[Display(Name = "WaterMark Size", Order = 1, GroupName = "Parameters")]
		public int WMSize
		{ get; set; }

		[Display(Name = "WaterMark Location", Description = "Select location", Order = 2, GroupName = "Parameters")]
		public WaterMarkLocation WaterMarkLocation
		{
			get { return waterMarkLocation; }
			set { waterMarkLocation = value; }
		}

		[NinjaScriptProperty]
		[Range(1, 100)]
		[Display(Name = "WaterMark Opacity", Description = "Enter Opacity % from 1 to 100", Order = 3, GroupName = "Parameters")]
		public int WMOpacity
		{ get; set; }

		[XmlIgnore]
		[Display(Name = "Text Color", Description = "Watermark Color", Order = 4, GroupName = "Parameters")]
		public System.Windows.Media.Brush WMBrush
		{
			get { return textBrush; }
			set { textBrush = value; }
		}

		[Browsable(false)]
		public string TextBrushSerialize
		{
			get { return Serialize.BrushToString(WMBrush); }
			set { WMBrush = Serialize.StringToBrush(value); }
		}


		[Display(Name = "WaterMark Style", Order = 5, GroupName = "Parameters", Description = "Select watermark style")]
		public WaterMarkStyle WaterMarkStyle
		{
			get { return waterMarkStyle; }
			set { waterMarkStyle = value; }
		}

		[NinjaScriptProperty]
		[Display(Name = "Custom Text", Description = "Used when Custom WaterMarkStyle is selected", Order = 6, GroupName = "Parameters")]
		public string CustomText
		{ get; set; }


		#endregion
	}
}
public enum WaterMarkStyle
{
	Symbol,
	SymbolAndExpiry,
	Description,
	Custom
}

public enum WaterMarkLocation
{
	Center,
	TopLeft,
	TopCenter,
	TopRight,
	BottomLeft,
	BottomCenter,
	BottomRight
}

#region NinjaScript generated code. Neither change nor remove.

namespace NinjaTrader.NinjaScript.Indicators
{
	public partial class Indicator : NinjaTrader.Gui.NinjaScript.IndicatorRenderBase
	{
		private j2.j2Watermark[] cachej2Watermark;
		public j2.j2Watermark j2Watermark(int wMSize, int wMOpacity, string customText)
		{
			return j2Watermark(Input, wMSize, wMOpacity, customText);
		}

		public j2.j2Watermark j2Watermark(ISeries<double> input, int wMSize, int wMOpacity, string customText)
		{
			if (cachej2Watermark != null)
				for (int idx = 0; idx < cachej2Watermark.Length; idx++)
					if (cachej2Watermark[idx] != null && cachej2Watermark[idx].WMSize == wMSize && cachej2Watermark[idx].WMOpacity == wMOpacity && cachej2Watermark[idx].CustomText == customText && cachej2Watermark[idx].EqualsInput(input))
						return cachej2Watermark[idx];
			return CacheIndicator<j2.j2Watermark>(new j2.j2Watermark(){ WMSize = wMSize, WMOpacity = wMOpacity, CustomText = customText }, input, ref cachej2Watermark);
		}
	}
}

namespace NinjaTrader.NinjaScript.MarketAnalyzerColumns
{
	public partial class MarketAnalyzerColumn : MarketAnalyzerColumnBase
	{
		public Indicators.j2.j2Watermark j2Watermark(int wMSize, int wMOpacity, string customText)
		{
			return indicator.j2Watermark(Input, wMSize, wMOpacity, customText);
		}

		public Indicators.j2.j2Watermark j2Watermark(ISeries<double> input , int wMSize, int wMOpacity, string customText)
		{
			return indicator.j2Watermark(input, wMSize, wMOpacity, customText);
		}
	}
}

namespace NinjaTrader.NinjaScript.Strategies
{
	public partial class Strategy : NinjaTrader.Gui.NinjaScript.StrategyRenderBase
	{
		public Indicators.j2.j2Watermark j2Watermark(int wMSize, int wMOpacity, string customText)
		{
			return indicator.j2Watermark(Input, wMSize, wMOpacity, customText);
		}

		public Indicators.j2.j2Watermark j2Watermark(ISeries<double> input , int wMSize, int wMOpacity, string customText)
		{
			return indicator.j2Watermark(input, wMSize, wMOpacity, customText);
		}
	}
}

#endregion
